#!/usr/bin/perl -w
#
# $Id: GKismetApplication.pm,v 1.47 2003/06/29 19:51:20 solovam Exp $
#
# GKismetApplication class:
#
package GKismetApplication;

use Gnome;
use strict;
use InfoBox;
use NetworkTree;
use StatusBar;
use Connection;
use ConnectionDialog;
use PrefsDialog;
use AboutDialog;
use Preferences;
use PacketWindow;
use StringWindow;
use Observable;
use Misc;

#
#
# Constructor
#
sub new
{
	my $type = shift;
	my $self = {};
	bless $self, $type;

	# Gnome init
	init Gnome 'gkismet';

	# Create app
	my $application = new Gnome::App("gkismet", "gkismet");
	$self->{'application'} = $application;
	$application->set_default_size(Gtk::Gdk->screen_width() * 0.6, Gtk::Gdk->screen_height() * 0.8);
	$application->signal_connect("delete_event", sub {Gtk->exit(0)});

	# Preferences
	my $preferences = new Preferences();
	$self->{'preferences'} = $preferences;
	
	# Status line
	my $statusBar = new StatusBar();
	$self->{'statusBar'} = $statusBar;

	# My info box
	my $infoBox = new InfoBox();
	$self->{'infoBox'} = $infoBox;

	# Ntree
	my $networkTree = new NetworkTree($self);
	$self->{'networkTree'} = $networkTree;
	$networkTree->addObserver($self);

	# Build menu info
	my @menuInfo =
	(
	{
		type => 'subtree', label => '_File', subtree =>
		[
			{type => 'item', label => '_Connect', pixmap_type => 'stock', pixmap_info => 'Menu_Open',
				callback => sub {$self->connect()}},
			{type => 'item', label => '_Disconnect', pixmap_type => 'stock', pixmap_info => 'Menu_Close',
				callback => sub {$self->disconnect()}},
			{type => 'separator'},
			{type => 'item', label => '_Quit', pixmap_type => 'stock', pixmap_info => 'Menu_Quit',
				callback => sub {Gtk->exit(0)}}
		]
	},
	{
		type => 'subtree', label => '_View', subtree =>
		[
			{type => 'item', label => '_Expand', pixmap_type => 'stock', pixmap_info => 'Menu_Book Open',
				callback => sub {$self->{'networkTree'}->expand()}},
			{type => 'item', label => '_Collapse', pixmap_type => 'stock', pixmap_info => 'Menu_Book Red',
				callback => sub {$self->{'networkTree'}->collapse()}},
			{type => 'item', label => '_Packet Dump', pixmap_type => 'stock', pixmap_info => 'Menu_Timer',
				callback => sub {$self->packetOpenSelected()}},
			{type => 'item', label => '_String Dump', pixmap_type => 'stock', pixmap_info => 'Menu_Timer',
				callback => sub {$self->stringOpenSelected()}},
			{type => 'subtree', label => 'Sort', subtree =>
				[
					{type => 'item', label => 'Ascending',
						callback => sub {$self->{'networkTree'}->setSortOrder('ascending')}},
					{type => 'item', label => 'Descending',
						callback => sub {$self->{'networkTree'}->setSortOrder('descending')}},
				]
			},
			{type => 'subtree', label => 'Sort Networks', subtree =>
				[
					{type => 'item', label => 'By channel',
						callback => sub {$self->{'networkTree'}->setNetworksSortBy('channel')}},
					{type => 'item', label => 'By first seen',
						callback => sub {$self->{'networkTree'}->setNetworksSortBy('firsttime')}},
					{type => 'item', label => 'By last seen',
						callback => sub {$self->{'networkTree'}->setNetworksSortBy('lasttime')}},
					{type => 'item', label => 'By BSSID',
						callback => sub {$self->{'networkTree'}->setNetworksSortBy('bssid')}},
					{type => 'item', label => 'By SSID',
						callback => sub {$self->{'networkTree'}->setNetworksSortBy('ssid')}},
					{type => 'item', label => 'By data packet count',
						callback => sub {$self->{'networkTree'}->setNetworksSortBy('datapackets')}},
					{type => 'item', label => 'By signal power',
						callback => sub {$self->{'networkTree'}->setNetworksSortBy('signal')}},
					{type => 'item', label => 'By WEP',
						callback => sub {$self->{'networkTree'}->setNetworksSortBy('wep')}},
				]
			},
			{type => 'subtree', label => 'Sort Clients', subtree =>
				[
					{type => 'item', label => 'By channel',
						callback => sub {$self->{'networkTree'}->setClientsSortBy('channel')}},
					{type => 'item', label => 'By first seen',
						callback => sub {$self->{'networkTree'}->setClientsSortBy('firsttime')}},
					{type => 'item', label => 'By last seen',
						callback => sub {$self->{'networkTree'}->setClientsSortBy('lasttime')}},
					{type => 'item', label => 'By MAC',
						callback => sub {$self->{'networkTree'}->setClientsSortBy('mac')}},
					{type => 'item', label => 'By data packet count',
						callback => sub {$self->{'networkTree'}->setClientsSortBy('datapackets')}},
					{type => 'item', label => 'By signal power',
						callback => sub {$self->{'networkTree'}->setClientsSortBy('signal')}},
					{type => 'item', label => 'By WEP',
						callback => sub {$self->{'networkTree'}->setClientsSortBy('wep')}},
				]
			}
		]
	},
	{
		type => 'subtree', label => '_Settings', subtree =>
		[
			{type => 'item', label => '_Preferences', pixmap_type => 'stock', pixmap_info => 'Menu_Preferences',
				callback => sub{$self->preferencesOpen()}}
		]
	},
	{
		type => 'subtree', label => '_Help', subtree =>
		[
			{type => 'item', label => '_About', pixmap_type => 'stock', pixmap_info => 'Menu_About',
				callback => sub{$self->aboutOpen()}}
		]
	}
	);

	# Build toolbar info
	my @toolbarInfo = (
		{type => 'item', label => 'Connect', pixmap_type => 'stock', pixmap_info => 'Open', 'hint' => 'Connect to Kismet server',
			callback => sub {$self->connect()}},
		{type => 'item', label => 'Disconnect', pixmap_type => 'stock', pixmap_info => 'Close', 'hint' => 'Disconnect from Kismet server',
			callback => sub {$self->disconnect()}},
		{type => 'item', label => 'Expand', pixmap_type => 'stock', pixmap_info => 'Book Open', 'hint' => 'Expand network node',
			callback => sub {$self->{'networkTree'}->expand()}},
		{type => 'item', label => 'Collapse', pixmap_type => 'stock', pixmap_info => 'Book Red', 'hint' => 'Collapse network node',
			callback => sub {$self->{'networkTree'}->collapse()}},
		{type => 'item', label => 'Packet Dump', pixmap_type => 'stock', pixmap_info => 'Timer', 'hint' => 'Open packet dump window',
			callback => sub {$self->packetOpenSelected()}},
		{type => 'item', label => 'String Dump', pixmap_type => 'stock', pixmap_info => 'Timer', 'hint' => 'Open string dump window',
			callback => sub {$self->stringOpenSelected()}},
	);

	# Pack wisgets
	my $scrolledWindow = new Gtk::ScrolledWindow(undef, undef);
	$scrolledWindow->set_policy('automatic', 'automatic');
	my $vbox = new Gtk::VBox($false, 0);
	$vbox->pack_start($infoBox->getWidget(), $false, $false, 10);
	my $hbox = new Gtk::HBox($false, 0);
	$hbox->pack_start($scrolledWindow, $true, $true, 0);
	$hbox->pack_end($vbox, $false, $false, 10);
	$scrolledWindow->add($networkTree->getWidget());

	# Assemble main window
	$application->create_menus(@menuInfo);
	$application->create_toolbar(@toolbarInfo);
	$application->set_statusbar($statusBar->getWidget());
	$application->set_contents($hbox);
	$application->show_all();

	# Get menu and toolbar item widgets
	$self->{'expandMenuItem'} = $self->findWidgetInMenu(\@menuInfo, '_Expand'); 
	$self->{'collapseMenuItem'} = $self->findWidgetInMenu(\@menuInfo, '_Collapse');
	$self->{'packetDumpMenuItem'} = $self->findWidgetInMenu(\@menuInfo, '_Packet Dump');
	$self->{'stringDumpMenuItem'} = $self->findWidgetInMenu(\@menuInfo, '_String Dump');
	$self->{'expandToolbarItem'} = $self->findWidgetInMenu(\@toolbarInfo, 'Expand');
	$self->{'collapseToolbarItem'} = $self->findWidgetInMenu(\@toolbarInfo, 'Collapse');
	$self->{'packetDumpToolbarItem'} = $self->findWidgetInMenu(\@toolbarInfo, 'Packet Dump');
	$self->{'stringDumpToolbarItem'} = $self->findWidgetInMenu(\@toolbarInfo, 'String Dump');
	for my $item (qw(expandMenuItem collapseMenuItem packetDumpMenuItem stringDumpMenuItem
		expandToolbarItem collapseToolbarItem packetDumpToolbarItem stringDumpToolbarItem))
	{
		$self->{$item}->set_sensitive($false);
	}

	return $self;
}

#
# Find widget with specific label
#
sub findWidgetInMenu
{
	my $self = shift;
	my $menu = shift;
	my $label = shift;

	for my $item (@{$menu})
	{
		my $widget = $self->findWidget($item, $label);
		if(defined $widget)
		{
			return $widget;
		}
	}
	return undef;
}

#
# Find widget for a menu/toolbar item. Plucked from Gnome-Perl samples
#
sub findWidget
{
	my $self = shift;
	my $item = shift;
	my $label = shift;
	my $subtree;

	# Array
	if(ref($item) eq 'ARRAY')
	{
		# If found, return
		if($item->[0] eq 'item' && $item->[1] eq $label)
		{
			return $item->[-1];
		}

		# Is there a subtree?
		if($item->[0] eq 'subtree')
		{
			$subtree = $item->[3];
		}
	}
	# Hash
	else
	{
		# If found, return
		if($item->{'type'} eq 'item' && $item->{'label'} eq $label)
		{
			return $item->{'widget'};
		}

		# Is there a subtree?
		if ($item->{type} eq 'radioitems')
		{
                        $subtree = $item->{'moreinfo'};
                }
		if($item->{'type'} eq 'subtree')
		{
			$subtree = $item->{'subtree'};
		}
	}

	# If there was a subtree
	if(defined $subtree)
	{
		for (@{$subtree})
		{
			my $widget = $self->findWidget($_, $label);
			if(defined $widget)
			{
				return $widget;
			}
		}
        }

	# Not found
	return undef;
}

#
# Connect to server
#
sub connect
{
	my $self = shift;
	my $host = shift;
	my $port = shift;

	if(defined $self->{'connection'})
	{
		$self->disconnect();
	}

	if(!defined $host || !defined $port)
	{
		my $connectionDialog = new ConnectionDialog($self);
		($host, $port) = $connectionDialog->runAndClose();
		if(!defined $host)
		{
			return(0);
		}
	}

	my $connection = new Connection($self, $host, $port);
	if(!defined $connection)
	{
		my $messagebox = new Gnome::MessageBox("Can't connect to kismet server at $host:$port", 'error', 'Button_Ok');
		$messagebox->run_and_close();
		return(0);
	}
	$self->{'connection'} = $connection;

	for my $observer ($self, $self->{'networkTree'}, $self->{'infoBox'}, $self->{'statusBar'})
	{
		$connection->addObserver($observer);
	}
	return(1);
}

#
# Disconnect
#
sub disconnect
{
	my $self = shift;

	if(!defined $self->{'connection'})
	{
		return;
	}

	for my $observer ($self->{'networkTree'}, $self->{'infoBox'}, $self->{'statusBar'})
	{
		$observer->flush();
	}
	for my $bssid (keys %{$self->{'packetWindow'}})
	{
		$self->packetClose($bssid);
	}
	for my $bssid (keys %{$self->{'stringWindow'}})
	{
		$self->stringClose($bssid);
	}
	$self->{'connection'}->disconnect();
	delete $self->{'connection'};
}

#
# Handle server quit
#
sub update
{
	my $self = shift;
	my $object = shift;
	my $data = shift;

	if($object->isa('Connection'))
	{
		my $connection = $object;
		if($data->{'updateType'} eq 'disconnected')
		{
			$self->disconnect();

			my $messagebox = new Gnome::MessageBox("Lost connection to kismet server at " . $connection->host() . ":" . $connection->port(),
				'error', 'Button_Ok');
			$messagebox->run_and_close();
		}
	}
	elsif($object->isa('NetworkTree'))
	{
		my $networkTree = $object;
		if($data->{'updateType'} eq 'selection')
		{
			if($self->{'networkTree'}->getSelectedNetworks())
			{
				for my $item (qw(expandMenuItem collapseMenuItem packetDumpMenuItem stringDumpMenuItem
					expandToolbarItem collapseToolbarItem packetDumpToolbarItem stringDumpToolbarItem))
				{
					$self->{$item}->set_sensitive($true);
				}
			}
			else
			{
				for my $item (qw(expandMenuItem collapseMenuItem packetDumpMenuItem stringDumpMenuItem
					expandToolbarItem collapseToolbarItem packetDumpToolbarItem stringDumpToolbarItem))
				{
					$self->{$item}->set_sensitive($false);
				}
			}
		}
	}
}

#
# Run the app
#
sub run
{
	my $self = shift;

	my ($progname) = ($0 =~ /([^\/]*)$/);
	my $usage = "$progname <host> [port]\n";

	if($#ARGV > 1)
	{
		print $usage;
		exit 1;
	}

	my $host = $ARGV[0];
	my $port = $ARGV[1] ? $ARGV[1] : 2501;

	$self->connect($host, $port);

	main Gtk;

	exit(0);
}

#
# Get widget
#
sub getWidget
{
	my $self = shift;
	return $self->{'application'};
}

#
# Show properties dialog
#
sub preferencesOpen
{
	my $self = shift;
	if(!defined $self->{'preferencesDialog'})
	{
		my $preferencesDialog = new PrefsDialog($self);
		$self->{'preferencesDialog'} = $preferencesDialog;
	}
}

#
# Close preferences dialog
#
sub preferencesClose
{
	my $self = shift;
	delete $self->{'preferencesDialog'};
}

#
# Show about dialog
#
sub aboutOpen
{
	my $self = shift;
	if(!defined $self->{'aboutDialog'})
	{
		my $aboutDialog = new AboutDialog($self);
		$self->{'aboutDialog'} = $aboutDialog;
	}
}

#
# Close about dialog
#
sub aboutClose
{
	my $self = shift;
	delete $self->{'aboutDialog'};
}

#
# Open packet window for the selected networks
#
sub packetOpenSelected
{
	my $self = shift;
	my @selected = $self->{'networkTree'}->getSelectedNetworks();
	for my $bssid (@selected)
	{
		$self->packetOpen($bssid);
	}
}

#
# Open packet window by bssid
#
sub packetOpen
{
	my $self = shift;
	my $bssid = shift;
	if(!defined $self->{'packetWindow'}{$bssid})
	{
		$self->{'packetWindow'}{$bssid} = new PacketWindow($self, $bssid, $self->{'networkTree'}->getSsid($bssid));
		$self->{'connection'}->addObserver($self->{'packetWindow'}{$bssid});
		$self->{'connection'}->enablePackets();
	}
	else
	{
		$self->{'packetWindow'}{$bssid}->getWidget()->window()->raise();
	}
}

#
# Close packet window for the selected networks
#
sub packetCloseSelected
{
	my $self = shift;
	my @selected = $self->{'networkTree'}->getSelectedNetworks();
	for my $bssid (@selected)
	{
		$self->packetClose($bssid);
	}
}

#
# Close packet window by bssid
#
sub packetClose
{
	my $self = shift;
	my $bssid = shift;
	if(defined $self->{'packetWindow'}{$bssid})
	{
		if(defined $self->{'connection'})
		{
			if(keys %{$self->{'packetWindow'}} == 1)
			{
				$self->{'connection'}->disablePackets();
			}
			$self->{'connection'}->removeObserver($self->{'packetWindow'}{$bssid});
		}
		$self->{'packetWindow'}{$bssid}->getWidget()->destroy();
		delete $self->{'packetWindow'}{$bssid};
	}
}

#
# Open string window for the selected networks
#
sub stringOpenSelected
{
	my $self = shift;
	my @selected = $self->{'networkTree'}->getSelectedNetworks();
	for my $bssid (@selected)
	{
		$self->stringOpen($bssid);
	}
}

#
# Open string window by bssid
#
sub stringOpen
{
	my $self = shift;
	my $bssid = shift;
	if(!defined $self->{'stringWindow'}{$bssid})
	{
		$self->{'stringWindow'}{$bssid} = new StringWindow($self, $bssid, $self->{'networkTree'}->getSsid($bssid));
		$self->{'connection'}->addObserver($self->{'stringWindow'}{$bssid});
		$self->{'connection'}->enableStrings();
	}
	else
	{
		$self->{'stringWindow'}{$bssid}->getWidget()->window()->raise();
	}
}

#
# Close string window for the selected networks
#
sub stringCloseSelected
{
	my $self = shift;
	my @selected = $self->{'networkTree'}->getSelectedNetworks();
	for my $bssid (@selected)
	{
		$self->stringClose($bssid);
	}
}

#
# Close string window by bssid
#
sub stringClose
{
	my $self = shift;
	my $bssid = shift;
	if(defined $self->{'stringWindow'}{$bssid})
	{
		if(defined $self->{'connection'})
		{
			if(keys %{$self->{'stringWindow'}} == 1)
			{
				$self->{'connection'}->disableStrings();
			}
			$self->{'connection'}->removeObserver($self->{'stringWindow'}{$bssid});
		}
		$self->{'stringWindow'}{$bssid}->getWidget()->destroy();
		delete $self->{'stringWindow'}{$bssid};
	}
}

1;
