#!/usr/bin/perl -w
#
# $Id: NetworkTree.pm,v 1.58 2003/06/29 05:43:39 solovam Exp $
#
# NetworkTree class
#
package NetworkTree;

use Gtk;
use strict;
use Misc;
use Observable;

#
# Inherit methods from Observable
#
@Connection::ISA = qw(Observable);

my %network_type = (network_ap => 0, network_adhoc => 1, network_probe => 2, network_turbocell => 3, network_data => 4, network_remove => 5);
my %address_type = (address_none => 0, address_factory => 1, address_udp => 2, address_arp => 3, address_tcp => 4, address_dhcp => 5, address_group => 6);
my %carrier_type = (carrier_unknown => 0, carrier_80211b => 1, carrier_80211a => 2, carrier_80211g => 3, carrier_80211 => 4);
my %client_type = (client_unknown => 0, client_fromds => 1, client_tods => 2, client_interds => 3, client_established => 4);
my %carier_type = (carrier_unknown => 0, carrier_80211b => 1, carrier_80211a => 2, carrier_80211g => 3, carrier_80211 => 4);
my $max_manuf_score = 8;
my $timerTimeout = 1000;
my $activeTimeout = 5;
my %networkFieldType = (channel => 'num', firsttime => 'num', lasttime => 'num', datapackets => 'num', signal => 'num', bssid => 'str',
	ssid => 'str', wep => 'str');
my %clientFieldType = (channel => 'num', firsttime => 'num', lasttime => 'num', datapackets => 'num', signal => 'num', mac => 'str', wep => 'str');

#
# Constructor
#
sub new
{
	my $type = shift;
	my $gKismetApplication = shift;
	my $self = {};
	bless $self, $type;

	$self->{'gKismetApplication'} = $gKismetApplication;

	# Create ctree
	$self->{'ctree'} = new Gtk::CTree(3, 0);
	$self->{'ctree'}->set_column_auto_resize(0, $true);
	$self->{'ctree'}->set_column_auto_resize(1, $true);
	$self->{'ctree'}->set_line_style('dotted');
	$self->{'ctree'}->set_indent(15); 
	$self->{'ctree'}->set_spacing(5);
	$self->{'ctree'}->set_selection_mode(1);
	$self->{'ctree'}->set_column_visibility(2, $false);
	$self->{'ctree'}->set_sort_column(2);
	$self->{'ctree'}->set_compare_func(sub {$self->compare(@_)});
	$self->{'ctree'}->signal_connect('tree_select_row', sub {$self->selectHandler(@_)});
	$self->{'ctree'}->signal_connect('tree_unselect_row', sub {$self->selectHandler(@_)});
	$self->{'ctree'}->show();

	# Read manufacturers data
	my ($mydir) = ($0 =~ /^(.*)\//);
	$self->{'apManuf'} = $self->readManufData($mydir . '/ap_manuf');
	$self->{'clManuf'} = $self->readManufData($mydir . '/client_manuf');

	# Get colors
        $self->buildColors();

	# Ask to notify us of the preferences changes
	$self->{'gKismetApplication'}->{'preferences'}->addObserver($self);

	# Popup menu
	my @menuItems =
	(
                {path => '/Expand', callback => sub {$self->expand()}},
                {path => '/Collapse', callback => sub {$self->collapse()} },
                {path => '/Packet Dump', callback => sub {$self->{'gKismetApplication'}->packetOpenSelected()}},
                {path => '/String Dump', callback => sub {$self->{'gKismetApplication'}->stringOpenSelected()}},
		{path => '/Sort', type  => '<Branch>'},
		{path => '/Sort/Ascending', callback  => sub {$self->setSortOrder('ascending')}},
		{path => '/Sort/Descending', callback => sub {$self->setSortOrder('descending')}},
		{path => '/Sort Networks', type  => '<Branch>'},
		{path => '/Sort Networks/By channel', callback => sub {$self->setNetworksSortBy('channel')}},
		{path => '/Sort Networks/By first seen', callback => sub {$self->setNetworksSortBy('firsttime')}},
		{path => '/Sort Networks/By last seen', callback => sub {$self->setNetworksSortBy('lasttime')}},
		{path => '/Sort Networks/By BSSID', callback => sub {$self->setNetworksSortBy('bssid')}},
		{path => '/Sort Networks/By SSID', callback => sub {$self->setNetworksSortBy('ssid')}},
		{path => '/Sort Networks/By data packet count', callback => sub {$self->setNetworksSortBy('datapackets')}},
		{path => '/Sort Networks/By signal power', callback => sub {$self->setNetworksSortBy('signal')}},
		{path => '/Sort Networks/By WEP', callback => sub {$self->setNetworksSortBy('wep')}},
		{path => '/Sort Clients', type  => '<Branch>'},
		{path => '/Sort Clients/By channel', callback => sub {$self->setClientsSortBy('channel')}},
		{path => '/Sort Clients/By first seen', callback => sub {$self->setClientsSortBy('firsttime')}},
		{path => '/Sort Clients/By last seen', callback => sub {$self->setClientsSortBy('lasttime')}},
		{path => '/Sort Clients/By MAC', callback => sub {$self->setClientsSortBy('mac')}},
		{path => '/Sort Clients/By data packet count', callback => sub {$self->setClientsSortBy('datapackets')}},
		{path => '/Sort Clients/By signal power', callback => sub {$self->setClientsSortBy('signal')}},
		{path => '/Sort Clients/By WEP', callback => sub {$self->setClientsSortBy('wep')}},
	);
	my $accelGroup = new Gtk::AccelGroup();
	my $itemFactory = new Gtk::ItemFactory('Gtk::Menu', '<main>', $accelGroup);
	$itemFactory->create_items(@menuItems);
	my $popup = $itemFactory->get_widget('<main>');
	$popup->show_all();
	$self->{'popup'} = $popup;
	$self->{'ctree'}->signal_connect('button_press_event', sub {$self->buttonPressHandler(@_)});

	# Sorting
	$self->{'ctree'}->set_sort_type(0);
	$self->{'networksSortBy'} = 'firsttime';
	$self->{'clientsSortBy'} = 'firsttime';

	# Set timer
	Gtk->timeout_add($timerTimeout, sub {$self->timerHandler()});

	# Pixmaps
	$self->{'pixmapsBuilt'} = $false;

	# Standard
	return $self;
}

#
# Set sorting order
#
sub setSortOrder
{
	my $self = shift;
	my $order = shift;
	if($order eq 'ascending')
	{
		$self->{'ctree'}->set_sort_type(0);
	}
	else
	{
		$self->{'ctree'}->set_sort_type(1);
	}
	$self->{'ctree'}->freeze();
	$self->{'ctree'}->sort_recursive(undef);
	$self->{'ctree'}->thaw();
}

#
# Handle tinmer event
#
sub timerHandler
{
	my $self = shift;
	$self->{'ctree'}->freeze();
	for my $bssid (keys %{$self->{'network'}})
	{
		my @info = $self->{'ctree'}->get_node_info($self->{'cnode'}{$bssid}{'node'});
		$self->{'ctree'}->set_node_info($self->{'cnode'}{$bssid}{'node'},
			$self->getNetworkFieldValue($bssid, 'SSID'), 5,
			$self->{'pixmap'}{$self->getNetworkPixmapName($bssid)}{'pixmap'},
			$self->{'pixmap'}{$self->getNetworkPixmapName($bssid)}{'mask'},
			$self->{'pixmap'}{$self->getNetworkPixmapName($bssid)}{'pixmap'},
			$self->{'pixmap'}{$self->getNetworkPixmapName($bssid)}{'mask'},
			$false, $info[7]);
 		for my $mac (keys %{$self->{'network'}{$bssid}{'client'}})
		{
			my @info = $self->{'ctree'}->get_node_info($self->{'cnode'}{$bssid}{'client'}{$mac}{'node'});
			$self->{'ctree'}->set_node_info($self->{'cnode'}{$bssid}{'client'}{$mac}{'node'},
				$self->getClientFieldValue($bssid, $mac, 'MAC'), 5,
				$self->{'pixmap'}{$self->getClientPixmapName($bssid, $mac)}{'pixmap'},
				$self->{'pixmap'}{$self->getClientPixmapName($bssid, $mac)}{'mask'},
				$self->{'pixmap'}{$self->getClientPixmapName($bssid, $mac)}{'pixmap'},
				$self->{'pixmap'}{$self->getClientPixmapName($bssid, $mac)}{'mask'},
				$false, $info[7]);
		}
	}
	$self->sortUpdated();
	$self->{'ctree'}->thaw();
	return $true;
}

#
# Sort updated items in ctree
#
sub sortUpdated
{
	my $self = shift;


	if($self->{'networkUpdateFlag'})
	{
		$self->{'ctree'}->sort(undef);
	}
	$self->{'networkUpdateFlag'} = $false;

	for my $bssid (keys %{$self->{'network'}})
        {
		if($self->{'clientUpdateFlag'}{$bssid})
		{
			$self->{'ctree'}->sort($self->{'cnode'}{$bssid}{'node'});
		}
		$self->{'clientUpdateFlag'}{$bssid} = $false;
	}
}

#
# Change network sorting
#
sub setNetworksSortBy
{
	my $self = shift;
	my $networksSortBy = shift;
	$self->{'ctree'}->freeze();
	$self->{'networksSortBy'} = $networksSortBy;
	$self->{'ctree'}->sort_recursive(undef);
	$self->{'ctree'}->thaw();
}

#
# Change client sorting
#
sub setClientsSortBy
{
	my $self = shift;
	my $clientsSortBy = shift;
	$self->{'ctree'}->freeze();
	$self->{'clientsSortBy'} = $clientsSortBy;
	$self->{'ctree'}->sort_recursive(undef);
	$self->{'ctree'}->thaw();
}

#
# Handle selection changes
#
sub selectHandler
{
	my $self = shift;
	$self->notifyObservers({'updateType' => 'selection'});
}

#
# Comparison function for CTree
#
sub compare
{
	my $self = shift;
	my $ctree = shift;
	my $text1= shift;
	my $text2 = shift;

	my($nodeType1, @fields1) = split /,/, $text1;
	my($nodeType2, @fields2) = split /,/, $text2;
	if($nodeType1 eq 'networkNode' && $nodeType2 eq 'networkNode')
	{
		my $bssid1 = $fields1[0];
		my $bssid2 = $fields2[0];
		if($networkFieldType{$self->{'networksSortBy'}} eq 'num')
		{
			return $self->{'network'}{$bssid1}{$self->{'networksSortBy'}} <=>
				$self->{'network'}{$bssid2}{$self->{'networksSortBy'}};
		}
		elsif($networkFieldType{$self->{'networksSortBy'}} eq 'str')
		{
			return $self->{'network'}{$bssid1}{$self->{'networksSortBy'}} cmp
				$self->{'network'}{$bssid2}{$self->{'networksSortBy'}};
		}
	}
	elsif ($nodeType1 eq 'clientNode' && $nodeType2 eq 'clientNode')
	{
		my($bssid1, $mac1) = @fields1[0,1];
		my($bssid2, $mac2) = @fields2[0,1];
		if($clientFieldType{$self->{'clientsSortBy'}} eq 'num')
		{
			return $self->{'network'}{$bssid1}{'client'}{$mac1}{$self->{'clientsSortBy'}} <=>
				$self->{'network'}{$bssid2}{'client'}{$mac2}{$self->{'clientsSortBy'}};
		}
		elsif($clientFieldType{$self->{'clientsSortBy'}} eq 'str')
		{
			return $self->{'network'}{$bssid1}{'client'}{$mac1}{$self->{'clientsSortBy'}} cmp
				$self->{'network'}{$bssid2}{'client'}{$mac2}{$self->{'clientsSortBy'}};
		}
	}

	return 0;
}

#
# Button press handler
#
sub buttonPressHandler
{
	my $self = shift;
	my $ctree = shift;
	my $event = shift;

	if($event->{'button'} == 3)
	{
		my($row, $column) = $ctree->get_selection_info($event->{'x'}, $event->{'y'});
		if(defined $row && defined $column)
		{
			$ctree->select_row($row, $column);
			$self->{'popup'}->popup(undef, undef, $event->{'time'}, $event->{'button'}, undef, undef);
			return $true;
		}
	}
	return $false;
}

#
# Read manufacturers data from files
#
sub readManufData
{
	my $self = shift;
	my $file = shift;
	my %manuf;
	my @fieldNames = qw(mac name model ssid chan ip);

	open(FILE, $file) or die "Can't open $file: $!\n";
	while(<FILE>)
	{
		my @fields = split '\t', $_;
		my $mac = $fields[0]; 
		$mac = substr($mac . ':00:00:00:00:00:00', 0, 17);
		for (my $i = 0; $i <= $#fields; $i++)
		{
			$manuf{$mac}{$fieldNames[$i]} = $fields[$i];
		}
	}
	close(FILE);
	return(\%manuf);
}

#
# Remove all stored data
#
sub flush
{
	my $self = shift;
	$self->screenFlush();
	delete $self->{'network'};
}

#
# Remove all screen data
#
sub screenFlush
{
	my $self = shift;

	# Remove all nodes from Ctree
	$self->{'ctree'}->freeze();
	for my $n (keys %{$self->{'cnode'}})
        {
                if(defined $self->{'cnode'}{$n}{'node'})
                {
                        $self->{'ctree'}->remove_node($self->{'cnode'}{$n}{'node'});
                }
        }
	$self->{'ctree'}->thaw();

	# Delete cnodes hash
	delete $self->{'cnode'};
}

#
# Get widget
#
sub getWidget
{
	my $self = shift;
	return $self->{'ctree'};
}

#
# Handle an update from an observer
#
sub update
{
	my $self = shift;
	my $object = shift;
	my $data = shift;

	if($object->isa('Connection'))
	{
		if($data->{'updateType'} eq 'network')
		{
			my $bssid = $data->{'update'}{'bssid'};
			for my $f (keys %{$data->{'update'}})
			{
				$self->{'network'}{$bssid}{$f} = $data->{'update'}{$f};
			}
			$self->{'networkUpdateFlag'} = $true;
			$self->networkScreenUpdate($bssid);
		}
		elsif($data->{'updateType'} eq 'client')
		{
			my $bssid = $data->{'update'}{'bssid'};
			my $mac = $data->{'update'}{'mac'};
			for my $f (keys %{$data->{'update'}})
			{
				$self->{'network'}{$bssid}{'client'}{$mac}{$f} = $data->{'update'}{$f};
			}
			$self->clientScreenUpdate($bssid, $mac);
		}
		elsif($data->{'updateType'} eq 'remove')
		{
			my $bssid = $data->{'update'}{'bssid'};
			if(defined $self->{'network'}{$bssid})
			{
				delete $self->{'network'}{$bssid};
			}
			$self->{'clientUpdateFlag'}{$bssid} = $true;
			$self->networkScreenRemove($bssid);
		}
	}
	elsif($object->isa('Preferences'))
	{
		$self->{'ctree'}->freeze();
		$self->screenFlush();
		for my $bssid (keys %{$self->{'network'}})
		{
			$self->networkScreenUpdate($bssid);
			for my $mac (keys %{$self->{'network'}{$bssid}{'client'}})
			{
				$self->clientScreenUpdate($bssid, $mac);
			}
		}
		$self->{'ctree'}->thaw();
	}
}

#
# Network is updated or added
#
sub networkScreenUpdate
{
	my $self = shift;
	my $bssid = shift;

	$self->{'ctree'}->freeze();

	# If there is no network node create it
	if(!defined $self->{'cnode'}{$bssid}{'node'})
	{
		my $n = $self->{'ctree'}->insert_node(undef, undef,
			[$self->getNetworkFieldValue($bssid, 'SSID'), '', "networkNode,$bssid"], 5,
			$self->{'pixmap'}{$self->getNetworkPixmapName($bssid)}{'pixmap'},
			$self->{'pixmap'}{$self->getNetworkPixmapName($bssid)}{'mask'},
			$self->{'pixmap'}{$self->getNetworkPixmapName($bssid)}{'pixmap'},
			$self->{'pixmap'}{$self->getNetworkPixmapName($bssid)}{'mask'},
			$false, $false);
		$self->{'ctree'}->node_set_foreground($n, $self->getNetworkColor($bssid));
		$self->{'ctree'}->node_set_selectable($n, $true);
		if(!$self->{'ctree'}->selection())
		{
			$self->{'ctree'}->select($n);
		}
		$self->{'cnode'}{$bssid}{'node'} = $n;
	}
	# Otherwise update it
	else
	{
		my @info = $self->{'ctree'}->get_node_info($self->{'cnode'}->{$bssid}{'node'});
		$self->{'ctree'}->set_node_info($self->{'cnode'}{$bssid}{'node'},
			$self->getNetworkFieldValue($bssid, 'SSID'), 5,
			$self->{'pixmap'}{$self->getNetworkPixmapName($bssid)}{'pixmap'},
			$self->{'pixmap'}{$self->getNetworkPixmapName($bssid)}{'mask'},
			$self->{'pixmap'}{$self->getNetworkPixmapName($bssid)}{'pixmap'},
			$self->{'pixmap'}{$self->getNetworkPixmapName($bssid)}{'mask'},
			$false, $info[7]);
	}

	# Loop through network fields
	for my $f ($self->{'gKismetApplication'}{'preferences'}->getNetworkFieldNames())
	{
		# If the field is shown
		if($self->{'gKismetApplication'}{'preferences'}->getNetworkFieldStatus($f))
		{
			# If the field node is not defined create it
			if(!defined $self->{'cnode'}{$bssid}{$f}{'node'})
			{
				my $n = $self->{'ctree'}->insert_node($self->{'cnode'}{$bssid}{'node'}, undef,
					[$f, $self->getNetworkFieldValue($bssid, $f), "networkNodeField,$bssid,$f"], 5,
					undef, undef, undef, undef, $true, $false); 
				$self->{'ctree'}->node_set_selectable($n, $false);
				$self->{'cnode'}{$bssid}{$f}{'node'} = $n;
			}
			# Otherwise update it
			else
			{
				$self->{'ctree'}->set_text($self->{'cnode'}{$bssid}{$f}{'node'}, 1,
					$self->getNetworkFieldValue($bssid, $f));
			}
		}
	}

	$self->{'ctree'}->thaw();
}

#
# Network is deleted
#
sub networkScreenRemove
{
	my $self = shift;
	my $bssid = shift;
	if(defined $self->{'cnode'}{$bssid}{'node'})
	{
		$self->{'ctree'}->remove_node($self->{'cnode'}{$bssid}{'node'});
		delete $self->{'cnode'}{$bssid};
	}
}

#
# Client is updated or added
#
sub clientScreenUpdate
{
	my $self = shift;
	my $bssid = shift;
	my $mac = shift;

	$self->{'ctree'}->freeze();

	# If client node is not defined create it
	if(!defined $self->{'cnode'}{$bssid}{'client'}{$mac}{'node'})
	{
		my $n = $self->{'ctree'}->insert_node($self->{'cnode'}{$bssid}{'node'}, undef,
			[$self->getClientFieldValue($bssid, $mac, 'MAC'), '', "clientNode,$bssid,$mac"], 5,
			$self->{'pixmap'}{$self->getClientPixmapName($bssid, $mac)}{'pixmap'},
			$self->{'pixmap'}{$self->getClientPixmapName($bssid, $mac)}{'mask'},
			$self->{'pixmap'}{$self->getClientPixmapName($bssid, $mac)}{'pixmap'},
			$self->{'pixmap'}{$self->getClientPixmapName($bssid, $mac)}{'mask'},
			$false, $false);
		$self->{'cnode'}{$bssid}{'client'}{$mac}{'node'} = $n;
		$self->{'ctree'}->node_set_selectable($n, $false);
		$self->networkScreenUpdate($bssid);
	}
	# otherwise update it
	else
	{
		my @info = $self->{'ctree'}->get_node_info($self->{'cnode'}{$bssid}{'client'}{$mac}{'node'});
		$self->{'ctree'}->set_node_info($self->{'cnode'}{$bssid}{'client'}{$mac}{'node'},
			$self->getClientFieldValue($bssid, $mac, 'MAC'), 5,
			$self->{'pixmap'}{$self->getClientPixmapName($bssid, $mac)}{'pixmap'},
			$self->{'pixmap'}{$self->getClientPixmapName($bssid, $mac)}{'mask'},
			$self->{'pixmap'}{$self->getClientPixmapName($bssid, $mac)}{'pixmap'},
			$self->{'pixmap'}{$self->getClientPixmapName($bssid, $mac)}{'mask'},
			$false, $info[7]);
	}

	# Loop through client fields
	for my $f ($self->{'gKismetApplication'}{'preferences'}->getClientFieldNames())
	{
		# If the field is shown
		if($self->{'gKismetApplication'}{'preferences'}->getClientFieldStatus($f))
		{
			# If the client field is not defined create it
			if(!defined $self->{'cnode'}{$bssid}{'client'}{$mac}{$f}{'node'})
			{
				my $n = $self->{'ctree'}->insert_node($self->{'cnode'}{$bssid}{'client'}{$mac}{'node'}, undef,
					[$f, $self->getClientFieldValue( $bssid, $mac, $f), "clientNodeField,$bssid,$mac,$f"], 5,
					undef, undef, undef, undef, $true, $false);
				$self->{'cnode'}{$bssid}{'client'}{$mac}{$f}{'node'} = $n;
				$self->{'ctree'}->node_set_selectable($n, $false);
			}
			# otherwise update it
			else
			{
				$self->{'ctree'}->set_text($self->{'cnode'}{$bssid}{'client'}{$mac}{$f}{'node'}, 1,
					$self->getClientFieldValue($bssid, $mac, $f));
			}
		}
	}

	$self->{'ctree'}->thaw();
}

#
# Get colors
#
sub buildColors
{
	my $self = shift;
	my $colormap = $self->{'ctree'}->get_colormap();

	my $colorYellow = Gtk::Gdk::Color->parse_color("gold3");
	$colorYellow = $colormap->color_alloc($colorYellow);
	$self->{'colorYellow'} = $colorYellow;

	my $colorGreen = Gtk::Gdk::Color->parse_color("green3");
	$colorGreen = $colormap->color_alloc($colorGreen);
	$self->{'colorGreen'} = $colorGreen;

	my $colorRed = Gtk::Gdk::Color->parse_color("red2");
	$colorRed = $colormap->color_alloc($colorRed);
	$self->{'colorRed'} = $colorRed;

	my $colorBlack = Gtk::Gdk::Color->parse_color("black");
	$colorBlack = $colormap->color_alloc($colorBlack);
	$self->{'colorBlack'} = $colorBlack;
}

#
# Collapse selected nodes
#
sub collapse
{
	my $self = shift;
	my (@selection) = $self->{'ctree'}->selection();
	for my $selected (@selection)
	{
		$self->{'ctree'}->collapse_recursive($selected);
	}
}

#
# Expand selected nodes
#
sub expand
{
	my $self = shift;
	my (@selection) = $self->{'ctree'}->selection();
	for my $selected (@selection)
	{
		$self->{'ctree'}->expand_recursive($selected);
	}
}

#
# Get selected network bssid
#
sub getSelectedNetworks
{
	my $self = shift;
        my (@selection) = $self->{'ctree'}->selection();
	my @networks;
	for my $selected (@selection)
	{
		my $data = $self->{'ctree'}->node_get_text($selected, 2);
		my @fields = split /,/, $data;
		if($fields[0] eq 'networkNode')
		{
			push @networks, $fields[1];
		}
	}
	return @networks;
}

#
# Get ssid by bssid
#
sub getSsid
{
	my $self = shift;
	my $bssid = shift;
	if(defined $self->{'network'}{$bssid})
	{
		return $self->{'network'}{$bssid}{'ssid'};
	}
	return undef;
}

#
# Return printable network data
#
sub getNetworkFieldValue
{
	my $self = shift;
	my $bssid = shift;
	my $field = shift;

	if($field eq 'SSID')
	{
		my $ssid = $self->{'network'}{$bssid}{'ssid'};
		if($self->{'network'}{$bssid}{'cloaked'})
		{
			$ssid = '<' . $ssid . '>';
		}
		return $ssid;
	}
	if($field eq 'BSSID')
	{
		return $self->{'network'}{$bssid}{'bssid'};
	}
	elsif($field eq 'Carrier')
	{
		if($self->{'network'}{$bssid}{'carrierset'} & (1 << $carrier_type{'carrier_80211b'}))
		{
			return 'IEEE 802.11b';
		}
		elsif($self->{'network'}{$bssid}{'carrierset'} & (1 << $carrier_type{'carrier_80211a'}))
		{
			return 'IEEE 802.11a';
		}
		elsif($self->{'network'}{$bssid}{'carrierset'} & (1 << $carrier_type{'carrier_80211g'}))
		{
			return 'IEEE 802.11g';
		}
		elsif($self->{'network'}{$bssid}{'carrierset'} & (1 << $carrier_type{'carrier_80211'}))
		{
			return 'IEEE 802.11';
		}
	}
	elsif($field eq 'Manufacturer')
	{
		my $m;
		if($self->{'network'}{$bssid}{'type'} == $network_type{'network_adhoc'} || $self->{'network'}{$bssid}{'type'} == $network_type{'network_probe'})
		{
			$m = $self->{'clManuf'};
		}
		else
		{
			$m = $self->{'apManuf'};
		}

		if(defined $m->{$self->{'network'}{$bssid}{'manufkey'}})
		{
			return $m->{$self->{'network'}{$bssid}{'manufkey'}}{'name'}
		}
		else
		{
			return('Unknown');
		}
	}
	elsif($field eq 'Model')
	{
		my $m;
		if($self->{'network'}{$bssid}{'type'} == $network_type{'network_adhoc'} || $self->{'network'}{$bssid}{'type'} == $network_type{'network_probe'})
		{
			$m = $self->{'clManuf'};
		}
		else
		{
			$m = $self->{'apManuf'};
		}

		if(defined $m->{$self->{'network'}{$bssid}{'manufkey'}})
		{
			return $m->{$self->{'network'}{$bssid}{'manufkey'}}{'model'}
		}
		else
		{
			return('Unknown');
		}
	}
	elsif($field eq 'Max rate')
	{
		return $self->{'network'}{$bssid}{'maxrate'};
	}
	elsif($field eq 'First seen')
	{
		return scalar localtime($self->{'network'}{$bssid}{'firsttime'});
	}
	elsif($field eq 'Last seen')
	{
		return scalar localtime($self->{'network'}{$bssid}{'lasttime'});
	}
	elsif($field eq 'Clients')
	{
		my @clients = keys %{$self->{'network'}{$bssid}{'client'}};
		return $#{clients} + 1;
	}
	elsif($field eq 'Type')
	{
		if($self->{'network'}{$bssid}{'type'} == $network_type{'network_ap'})
                {
                        return 'Access Point (infrastructure)';
                }
                elsif($self->{'network'}{$bssid}{'type'} == $network_type{'network_adhoc'})
                {
                        return 'Ad-hoc';
                }
                elsif($self->{'network'}{$bssid}{'type'} == $network_type{'network_probe'})
                {
                        return 'Probe request (searching client)';
                }
                elsif($self->{'network'}{$bssid}{'type'} == $network_type{'network_turbocell'})
                {
                        return 'Turbocell';
                }
                elsif($self->{'network'}{$bssid}{'type'} == $network_type{'network_data'})
                {
                        return 'Data (no network control traffic)';
                }
                elsif($self->{'network'}{$bssid}{'type'} == $network_type{'network_remove'})
                {
                        return 'Network remove request';
                }
	}
	elsif($field eq 'Channel')
	{
		return $self->{'network'}{$bssid}{'channel'};
	}
	elsif($field eq 'WEP')
	{
		$self->{'network'}{$bssid}{'wep'} ? return 'Yes' : return 'No';
	}
	elsif($field eq 'Beacon')
	{
		return $self->{'network'}{$bssid}{'beaconrate'};
	}
	elsif($field eq 'Data packets')
	{
		return $self->{'network'}{$bssid}{'datapackets'};
	}
	elsif($field eq 'LLC packets')
	{
		return $self->{'network'}{$bssid}{'llcpackets'};
	}
	elsif($field eq 'Crypt packets')
	{
		return $self->{'network'}{$bssid}{'cryptpackets'};
	}
	elsif($field eq 'Weak packets')
	{
		return $self->{'network'}{$bssid}{'weakpackets'};
	}
	elsif($field eq 'Data')
	{
		return $self->{'network'}{$bssid}{'datasize'} . " bytes";
	}
	elsif($field eq 'Quality')
	{
		return $self->{'network'}{$bssid}{'quality'} . "/" . $self->{'network'}{$bssid}{'bestquality'};
	}
	elsif($field eq 'Power')
	{
		return $self->{'network'}{$bssid}{'signal'} . "/" . $self->{'network'}{$bssid}{'bestsignal'};
	}
	elsif($field eq 'Noise')
	{
		return $self->{'network'}{$bssid}{'noise'} . "/" . $self->{'network'}{$bssid}{'bestnoise'};
	}
	elsif($field eq 'IP Range')
	{
		return $self->{'network'}{$bssid}{'rangeip'};
	}
	elsif($field eq 'IP Type')
	{
		if($self->{'network'}{$bssid}{'atype'} == $address_type{'address_none'})
                {
                        return 'None detected';
                }
                elsif($self->{'network'}{$bssid}{'atype'} == $address_type{'address_factory'})
                {
                        return 'Factory default';
                }
                elsif($self->{'network'}{$bssid}{'atype'} == $address_type{'address_udp'})
                {
                        return 'UDP (' .  $self->{'network'}{$bssid}{'octets'} . " octets)";
                }
                elsif($self->{'network'}{$bssid}{'atype'} == $address_type{'address_arp'})
                {
                        return 'ARP (' .  $self->{'network'}{$bssid}{'octets'} . " octets)";
                }
                elsif($self->{'network'}{$bssid}{'atype'} == $address_type{'address_tcp'})
                {
                        return 'TCP (' .  $self->{'network'}{$bssid}{'octets'} . " octets)";
                }
                elsif($self->{'network'}{$bssid}{'atype'} == $address_type{'address_dhcp'})
                {
                        return 'DHCP';
                }
                elsif($self->{'network'}{$bssid}{'atype'} == $address_type{'address_group'})
                {
                        return 'Group (aggregate)';
                }
                else
                {
                        return 'UNKNOWN';
                }
	}
	else
	{
		return('Not implemented');
	}
}

#
# Return printable client data
#
sub getClientFieldValue
{
	my $self = shift;
	my $bssid = shift;
	my $mac = shift;
	my $field = shift;

	if($field eq 'MAC')
	{
		return $self->{'network'}{$bssid}{'client'}{$mac}{'mac'};
	}
	elsif($field eq 'Type')
	{
		if($self->{'network'}{$bssid}{'client'}{$mac}{'type'} == $client_type{'client_unknown'})
		{
			return 'Unknown';
		}
		elsif ($self->{'network'}{$bssid}{'client'}{$mac}{'type'} == $client_type{'client_fromds'})
		{
			return 'From Distribution (AP->Wireless)';
		}
		elsif ($self->{'network'}{$bssid}{'client'}{$mac}{'type'} == $client_type{'client_tods'})
		{
			return 'To Distribution (Wireless->AP)';
		}
		elsif ($self->{'network'}{$bssid}{'client'}{$mac}{'type'} == $client_type{'client_interds'})
		{
			return 'Intra-distribution (AP->AP)';
		}
		elsif ($self->{'network'}{$bssid}{'client'}{$mac}{'type'} == $client_type{'client_established'})
		{
			return 'Established (Wireless->AP and AP->Wireless)';
		}
		else
		{
			return 'Unknown';
		}
	}
	elsif($field eq 'Manufacturer')
	{
		if(defined $self->{'clManuf'}{$self->{'network'}{$bssid}{'client'}{$mac}{'manufkey'}})
		{
			return $self->{'clManuf'}{$self->{'network'}{$bssid}{'client'}{$mac}{'manufkey'}}{'name'};
		}
		else
		{
			return('Unknown');
		}
	}
	elsif($field eq 'Model')
	{
		if(defined $self->{'clManuf'}{$self->{'network'}{$bssid}{'client'}{$mac}{'manufkey'}})
		{
			return $self->{'clManuf'}{$self->{'network'}{$bssid}{'client'}{$mac}{'manufkey'}}{'model'}
		}
		else
		{
			return('Unknown');
		}
	}
	elsif($field eq 'First seen')
	{
		return scalar localtime $self->{'network'}{$bssid}{'client'}{$mac}{'firsttime'};
	}
	elsif($field eq 'Last seen')
	{
		return scalar localtime $self->{'network'}{$bssid}{'client'}{$mac}{'lasttime'};
	}
	elsif($field eq 'Max rate')
	{
		return $self->{'network'}{$bssid}{'client'}{$mac}{'maxrate'};
	}
	elsif($field eq 'Channel')
	{
		return $self->{'network'}{$bssid}{'channel'};
	}
	elsif($field eq 'WEP')
	{
		$self->{'network'}{$bssid}{'client'}{$mac}{'wep'} ? return 'Yes' : return 'No';
	}
	elsif($field eq 'IP')
	{
		$self->{'network'}{$bssid}{'client'}{$mac}{'ip'};
	}
	elsif($field eq 'Data packets')
	{
		$self->{'network'}{$bssid}{'client'}{$mac}{'datapackets'};
	}
	elsif($field eq 'Crypt packets')
	{
		$self->{'network'}{$bssid}{'client'}{$mac}{'cryptpackets'};
	}
	elsif($field eq 'Weak packets')
	{
		$self->{'network'}{$bssid}{'client'}{$mac}{'weakpackets'};
	}
	elsif($field eq 'Data')
	{
		$self->{'network'}{$bssid}{'client'}{$mac}{'datasize'} . " bytes";
	}
	elsif($field eq 'Quality')
	{
		return $self->{'network'}{$bssid}{'client'}{$mac}{'quality'} . "/" . $self->{'network'}{$bssid}{'client'}{$mac}{'bestquality'};
	}
	elsif($field eq 'Power')
	{
		return $self->{'network'}{$bssid}{'client'}{$mac}{'signal'} . "/" . $self->{'network'}{$bssid}{'client'}{$mac}{'bestsignal'};
	}
	elsif($field eq 'Noise')
	{
		return $self->{'network'}{$bssid}{'client'}{$mac}{'noise'} . "/" . $self->{'network'}{$bssid}{'client'}{$mac}{'bestnoise'};
	}
	else
	{
		return('Not implemented');
	}
}

#
# Return color for a network
#
sub getNetworkColor
{
	my $self = shift;
	my $bssid = shift;

	if($self->{'network'}{$bssid}{'wep'} == 0)
	{
		if($self->{'network'}{$bssid}{'manufscore'} == $max_manuf_score)
		{
			return($self->{'colorRed'});
		}
		return($self->{'colorYellow'});
	}
	elsif($self->{'network'}{$bssid}{'wep'} == 1)
	{
		return($self->{'colorGreen'});
	}
	else
	{
		return($self->{'colorBlack'});
	}
}

#
# Get network pixmap
#
sub getNetworkPixmapName
{
	my $self = shift;
	my $bssid = shift;
 
	$self->buildPixmaps();
	my $pixmapName;

	if($self->{'network'}{$bssid}{'type'} == $network_type{'network_ap'})
        {
		$pixmapName = 'networkAP';
	}
	elsif($self->{'network'}{$bssid}{'type'} == $network_type{'network_adhoc'})
	{
		$pixmapName = 'networkAdhoc';
	}
	elsif($self->{'network'}{$bssid}{'type'} == $network_type{'network_probe'})
	{
		$pixmapName = 'networkProbe';
	}
	elsif($self->{'network'}{$bssid}{'type'} == $network_type{'network_turbocell'})
	{
		$pixmapName = 'networkTurbocell';
	}
	elsif($self->{'network'}{$bssid}{'type'} == $network_type{'network_data'})
	{
		$pixmapName = 'networkData';
  	}
	elsif($self->{'network'}{$bssid}{'type'} == $network_type{'network_remove'})
	{
		$pixmapName = 'networkRemove';
	}

	if(time() - $self->{'network'}{$bssid}{'lasttime'} < $activeTimeout)
	{
		$pixmapName .= 'Active';
	}

	return $pixmapName;
}
 
#
# Get client pixmap
#
sub getClientPixmapName
{
	my $self = shift;
	my $bssid = shift;
	my $mac = shift;
 
	$self->buildPixmaps();
	my $pixmapName;

	if($self->{'network'}{$bssid}{'client'}{$mac}{'type'} == $client_type{'client_unknown'})
	{
		$pixmapName = 'clientUnknown';
	}
	elsif ($self->{'network'}{$bssid}{'client'}{$mac}{'type'} == $client_type{'client_fromds'})
	{
		$pixmapName = 'clientFromDist';
	}
	elsif ($self->{'network'}{$bssid}{'client'}{$mac}{'type'} == $client_type{'client_tods'})
	{
		$pixmapName = 'clientToDist';
	}
	elsif ($self->{'network'}{$bssid}{'client'}{$mac}{'type'} == $client_type{'client_interds'})
	{
		$pixmapName = 'clientIntraDist';
	}
	elsif ($self->{'network'}{$bssid}{'client'}{$mac}{'type'} == $client_type{'client_established'})
	{
		$pixmapName = 'clientEstablished';
	}

	if(time() - $self->{'network'}{$bssid}{'client'}{$mac}{'lasttime'} < $activeTimeout)
	{
		$pixmapName .= 'Active';
	}

	return $pixmapName;
}

#
# Build pixmaps if not yet built
#
sub buildPixmaps
{
	my $self = shift;
	my ($mydir) = ($0 =~ /^(.*)\//);
	if(!$self->{'pixmapsBuilt'})
	{
		for my $pixmap (qw(networkAP networkAdhoc networkProbe networkTurbocell networkData networkRemove 
			clientFromDist clientToDist clientIntraDist clientEstablished clientUnknown
			networkAPActive NetworkAdhocActive networkProbeActive networkTurbocellActive networkDataActive networkRemoveActive
			clientFromDistActive clientToDistActive clientIntraDistActive clientEstablishedActive clientUnknownActive))
		{
			($self->{'pixmap'}{$pixmap}{'pixmap'}, $self->{'pixmap'}{$pixmap}{'mask'}) =
				Gtk::Gdk::Pixmap->create_from_xpm($self->{'ctree'}->window(),
				Gtk::Widget->get_default_style()->bg('normal'),
				$mydir . '/' . $pixmap . '.xpm');
		}
		$self->{'pixmapsBuilt'} = $true;
	}
}

1;
