#!/usr/bin/perl -w
#
# $Id: Preferences.pm,v 1.12 2003/06/28 19:01:06 solovam Exp $
#
# Preferences class
#
package Preferences;

use strict;
use Misc;
use Observable;

@Preferences::ISA = qw(Observable);

#
# Constructor
#
sub new
{
	my $type = shift;
	my $self = new Observable;
	bless $self, $type;

	# Network fields
	$self->{'networkFields'} =
		[
			{field => 'BSSID', show => 1},
			{field => 'Carrier', show => 1},
			{field => 'Manufacturer', show => 1},
			{field => 'Model', show => 1},
			{field => 'Max rate', show => 1},
			{field => 'First seen', show => 1},
			{field => 'Last seen', show => 1},
			{field => 'Clients', show => 1},
			{field => 'Type', show => 1},
			{field => 'Channel', show => 1},
			{field => 'WEP', show => 1},
			{field => 'Beacon',  show => 1},
			{field => 'Data packets', show => 1},
			{field => 'LLC packets', show => 1},
			{field => 'Crypt packets', show => 1},
			{field => 'Weak packets', show => 1},
			{field => 'Data', show => 1},
			{field => 'Quality', show => 1},
			{field => 'Power', show => 1},
			{field => 'Noise', show => 1},
			{field => 'IP Range', show => 1},
			{field => 'IP Type', show => 1}
		];
	for my $f (@{$self->{'networkFields'}})
	{
		$f->{'show'} = Gnome::Config->get_bool('/gkismet/networkFields/' . $f->{'field'} . '=true');
	}
	# Client fields
	$self->{'clientFields'} =
		[
			{field => 'Type', show => 1},
			{field => 'Manufacturer', show => 1},
			{field => 'Model', show => 1},
			{field => 'First seen', show => 1},
			{field => 'Last seen', show => 1},	
			{field => 'Max rate', show => 1},
			{field => 'Channel', show => 1},
			{field => 'WEP', show => 1},
			{field => 'IP', show => 1},
			{field => 'Data packets', show => 1},
			{field => 'Crypt packets', show => 1},
			{field => 'Weak packets', show => 1},
			{field => 'Data', show => 1},
			{field => 'Quality', show => 1},
			{field => 'Power', show => 1},
			{field => 'Noise', show => 1}
		];
	for my $f (@{$self->{'clientFields'}})
	{
		$f->{'show'} = Gnome::Config->get_bool('/gkismet/clientFields/' . $f->{'field'} . '=true');
	}
	# Packet window depth
	$self->{'packetDepth'} = Gnome::Config->get_int('/gkismet/clientFields/packetDepth=100');
	# String window depth
	$self->{'stringDepth'} = Gnome::Config->get_int('/gkismet/clientFields/stringDepth=100');

	return $self;
}

#
# Save preferences
#
sub save
{
	my $self = shift;

	for my $f (@{$self->{'networkFields'}})
	{
		Gnome::Config->set_bool('/gkismet/networkFields/' . $f->{'field'}, $f->{'show'});
	}
	for my $f (@{$self->{'clientFields'}})
	{
		Gnome::Config->set_bool('/gkismet/clientFields/' . $f->{'field'}, $f->{'show'});
	}
	Gnome::Config->set_int('/gkismet/clientFields/packetDepth', $self->{'packetDepth'});
	Gnome::Config->set_int('/gkismet/clientFields/stringDepth', $self->{'stringDepth'});
	Gnome::Config->sync();
	$self->notifyObservers($self);
}

#
# Get list of network field names
#
sub getNetworkFieldNames
{
	my $self = shift;
	my @fieldNames;
	for my $f (@{$self->{'networkFields'}})
	{
		push @fieldNames, $f->{'field'};
	}
	return @fieldNames;
}

#
# Get list of client field names
#
sub getClientFieldNames
{
	my $self = shift;
	my @fieldNames;
	for my $f (@{$self->{'clientFields'}})
	{
		push @fieldNames, $f->{'field'};
	}
	return @fieldNames;
}

#
# Get status of a network field
#
sub getNetworkFieldStatus
{
	my $self = shift;
	my $field = shift;
	for my $f (@{$self->{'networkFields'}})
	{
		if($f->{'field'} eq $field)
		{
			return $f->{'show'};
		}
	}
	return 0;
}

#
# Set status of a network field
#
sub setNetworkFieldStatus
{
	my $self = shift;
	my $field = shift;
	my $status = shift;
	for my $f (@{$self->{'networkFields'}})
	{
		if($f->{'field'} eq $field)
		{
			$f->{'show'} = $status;
		}
	}
	return 0;
}

#
# Get status of a client field
#
sub getClientFieldStatus
{
	my $self = shift;
	my $field = shift;
	for my $f (@{$self->{'clientFields'}})
	{
		if($f->{'field'} eq $field)
		{
			return $f->{'show'};
		}
	}
	return 0;
}

#
# Set status of a client field
#
sub setClientFieldStatus
{
	my $self = shift;
	my $field = shift;
	my $status = shift;
	for my $f (@{$self->{'clientFields'}})
	{
		if($f->{'field'} eq $field)
		{
			$f->{'show'} = $status;
		}
	}
	return 0;
}

#
# Get packet window depth
#
sub getPacketDepth
{
	my $self = shift;
	return $self->{'packetDepth'};
}

#
# Get string window depth
#
sub getStringDepth
{
	my $self = shift;
	return $self->{'stringDepth'};
}

#
# Set packet window depth
#
sub setPacketDepth
{
	my $self = shift;
	my $depth = shift;
	$self->{'packetDepth'} = $depth;
}

#
# Set string window depth
#
sub setStringDepth
{
	my $self = shift;
	my $depth = shift;
	$self->{'stringDepth'} = $depth;
}

1;
