#!/usr/bin/perl -w
#
# $Id: PrefsDialog.pm,v 1.15 2003/06/29 05:34:04 solovam Exp $
#
# PrefsDialog class
#
package PrefsDialog;

use Gnome;
use strict;
use Misc;

#
# Constructor
#
sub new
{
	my $type = shift;
	my $self = {};
	bless $self, $type;

	my $gKismetApplication = shift;
	$self->{'gKismetApplication'} = $gKismetApplication;
	my $preferences = $gKismetApplication->{'preferences'};

	my $propertyBox = new Gnome::PropertyBox();
	$self->{'propertyBox'} = $propertyBox;
	$propertyBox->signal_connect('destroy', sub{$self->destroyHandler(@_)});
	$propertyBox->signal_connect('apply', sub {$self->applyHandler(@_)});
	$propertyBox->set_parent($gKismetApplication->getWidget());

	# Page 0
	my $nNetworkFields = scalar $preferences->getNetworkFieldNames();
	my $networkFrame = new Gtk::Frame('Network');
	my $networkTable = new Gtk::Table($nNetworkFields + 1, 2, $false);
	$networkFrame->add($networkTable);
	my $i = 0;
	for my $f ($preferences->getNetworkFieldNames())
	{
		my $button = new Gtk::CheckButton($f);
		$button->set_active($preferences->getNetworkFieldStatus($f));
		$self->{'networkFieldButton'}{$f} = $button;
		$button->signal_connect('clicked', sub {$self->changedHandler()});
		if(!($i % 2))
		{
			$networkTable->attach($button, 0, 1, $i, $i + 1, 'fill', 'fill', 0, 0);
		}
		else
		{
			$networkTable->attach($button, 1, 2, $i - 1, $i, 'fill', 'fill', 0, 0);
		}
		$i++;
	}
	my $nClientFields = scalar $preferences->getClientFieldNames();
	my $clientFrame = new Gtk::Frame('Client');
	my $clientTable = new Gtk::Table($nClientFields + 1, 2, $false);
	$clientFrame->add($clientTable);
	$i = 0;
	for my $f ($preferences->getClientFieldNames())
	{
		my $button = new Gtk::CheckButton($f);
		$button->set_active($preferences->getClientFieldStatus($f));
		$self->{'clientFieldButton'}{$f} = $button;
		$button->signal_connect('clicked', sub {$self->changedHandler()});
		if(!($i % 2))
		{
			$clientTable->attach($button, 0, 1, $i, $i + 1, 'fill', 'fill', 0, 0);
		}
		else
		{
			$clientTable->attach($button, 1, 2, $i - 1, $i, 'fill', 'fill', 0, 0);
		}
		$i++;
	}
	my $hbox = new Gtk::HBox($true, 0);
	$hbox->pack_start($networkFrame, $true, $false, 0);
	$hbox->pack_start($clientFrame, $true, $false, 0);
	my $tabLabel0 = new Gtk::Label('Fields');
	$propertyBox->append_page($hbox, $tabLabel0);

	# Page 1
	my $packetAdjustment = new Gtk::Adjustment($preferences->getPacketDepth(), 100, 10000, 100, 1000, 0);
	my $packetSpinButton = new Gtk::SpinButton($packetAdjustment, 0, 0);
	$packetSpinButton->set_numeric($true);
	$packetSpinButton->signal_connect('changed', sub {$self->changedHandler(@_)});
	$self->{'packetSpinButton'} = $packetSpinButton;
	my $stringAdjustment = new Gtk::Adjustment($preferences->getStringDepth(), 100, 10000, 100, 1000, 0);
	my $stringSpinButton = new Gtk::SpinButton($stringAdjustment, 0, 0);
	$stringSpinButton->set_numeric($true);
	$stringSpinButton->signal_connect('changed', sub {$self->changedHandler(@_)});
	$self->{'stringSpinButton'} = $stringSpinButton;
	my $packetLabel = new Gtk::Label('Packet window depth:');
	my $stringLabel = new Gtk::Label('String window depth:');
	my $packetHBox = new Gtk::HBox($true, 0);
	my $stringHBox= new Gtk::HBox($true, 0);
	$packetHBox->pack_start($packetLabel, $true, $true, 20);
	$packetHBox->pack_start($packetSpinButton, $true, $true, 20);
	$stringHBox->pack_start($stringLabel, $true, $true, 20);
	$stringHBox->pack_start($stringSpinButton, $true, $true, 20);
	my $vbox = new Gtk::VBox($false, 0);
	$vbox->pack_start($packetHBox, $false, $false, 20);
	$vbox->pack_start($stringHBox, $false, $false, 20);
	my $depthFrame = new Gtk::Frame('Window depth');
	$depthFrame->add($vbox);
	my $tabLabel1 = new Gtk::Label('Window depth');
	$propertyBox->append_page($depthFrame, $tabLabel1);

	# Show widget
	$propertyBox->show_all();

	return $self;
}

#
# Destroy handler
#
sub destroyHandler
{
	my $self = shift;
	$self->{'gKismetApplication'}->preferencesClose();
}

#
# Handle a change to a widget
#
sub changedHandler
{
	my $self = shift;
	$self->{'propertyBox'}->changed();
	return($false);
}

#
# Applied handler
#
sub applyHandler
{
	my $self = shift;
	my $widget = shift;
	my $page = shift;

	if($page != -1)
	{
		return($false);
	}

	my $preferences = $self->{'gKismetApplication'}->{'preferences'};
	for my $f ($preferences->getNetworkFieldNames())
	{
		$preferences->setNetworkFieldStatus($f, $self->{'networkFieldButton'}{$f}->get_active());
	}
	for my $f ($preferences->getClientFieldNames())
	{
		$preferences->setClientFieldStatus($f, $self->{'clientFieldButton'}{$f}->get_active());
	}
	$preferences->setPacketDepth($self->{'packetSpinButton'}->get_value_as_int());
	$preferences->setStringDepth($self->{'stringSpinButton'}->get_value_as_int());
	$preferences->save();
	return($false);
}

#
# Get widget
#
sub getWidget
{
	my $self = shift;
	return $self->{'propertyBox'};
}

1;
