#!/usr/bin/perl -w
#
# $Id: StringWindow.pm,v 1.14 2003/06/29 20:54:21 solovam Exp $
#
# StringWindow class
#
package StringWindow;

use Gtk;
use Misc;
use strict;

#
# Constructor
#
sub new
{
	my $type = shift;
	my $self = {};
	bless $self, $type;
	$self->{'gKismetApplication'} = shift;
	$self->{'bssid'} = shift;
	$self->{'ssid'} = shift;

	# Main string dump window
	my $window = new Gtk::Window('dialog');
	$window->signal_connect("delete_event", sub {$self->deleteHandler()});
	$window->set_default_size(Gtk::Gdk->screen_width() * 0.8, Gtk::Gdk->screen_height() * 0.5);
	$window->set_title('String Dump ' . $self->{'ssid'} . ' ' . $self->{'bssid'});
	$self->{'window'} = $window;

	# Scrolled window
	my $scrolledWindow = new Gtk::ScrolledWindow(undef, undef);

	# Clist
	my @titles = ('BSSID', 'Source MAC', 'Text');
	my $clist = new_with_titles Gtk::CList(@titles);
	for(my $i = 0; $i <= $#titles; $i++)
	{
		$clist->set_column_auto_resize($i, $true);
	}
	$self->{'clist'} = $clist;
	$self->{'clist'}->set_selectable($false);
	$self->{'nRows'} = 0;

	# Buttonbox, buttons
	my $bbox = new Gtk::HButtonBox();
	$bbox->set_layout('spread');
	$bbox->set_spacing(5);
	my $buttonClose = new Gtk::Button("Close");
	$buttonClose->signal_connect( "clicked", sub {$self->closeClicked()});
	$bbox->add($buttonClose);
	my $buttonPause = new Gtk::Button("Pause");
	$buttonPause->signal_connect( "clicked", sub {$self->pauseClicked($buttonPause)});
	$bbox->add($buttonPause);

	# Assemble all
	$scrolledWindow->add($clist);
	$self->{'scrolledWindow'} = $scrolledWindow;
	my $vbox = new Gtk::VBox($false, 0);
	$vbox->pack_start($scrolledWindow, $true, $true, 0);
	$vbox->pack_end($bbox, $false, $false, 10);
	$window->add($vbox);
	$window->show_all();

	# Misc
	$self->{'paused'} = $false;

	return $self;
}

#
# Delete signal handler
#
sub deleteHandler
{
	my $self = shift;
	$self->{'gKismetApplication'}->stringClose($self->{'bssid'});
	return($false);
}

#
# Close button clicked
#
sub closeClicked
{
	my $self = shift;
	$self->{'gKismetApplication'}->stringClose($self->{'bssid'});
	return($true);
}

#
# Pause button clicked
#
sub pauseClicked
{
	my $self = shift;
	my $button = shift;
	if($self->{'paused'} == $false)
	{
		$self->{'paused'} = $true;
		$button->child()->set("Resume");
	}
	else
	{
		$self->{'paused'} = $false;
		my $label = new Gtk::Label('Pause');
		$button->child()->set("Pause");
	}
	return($true);
}

#
# Get widget
#
sub getWidget
{ 
	my $self = shift;
	return $self->{'window'};
}

#
# Remove all stored data
#
sub flush
{
	my $self = shift;
	$self->{'clist'}->clear();
}

#
# Handle an update from an observable
#
sub update
{
	my $self = shift;
	my $object = shift;
	my $data = shift;

	if($object->isa('Connection'))
	{
		if($data->{'updateType'} eq 'string' && $data->{'update'}{'bssid'} eq $self->{'bssid'} && $self->{'paused'} == $false)
		{
			my $string = $data->{'update'};
			my $bssid = $string->{'bssid'};
			my $srcmac = $string->{'sourcemac'};
			my $text = $string->{'text'};

			$self->{'clist'}->freeze();

			my $adjustment = $self->{'scrolledWindow'}->get_vadjustment();
                        my $value = $adjustment->get_value();
                        my $page_size = $adjustment->page_size();
                        my $step_increment = $adjustment->step_increment();
                        my $upper = $adjustment->upper();

			$self->{'clist'}->append($bssid, $srcmac, $text);
			$self->{'nRows'}++;
                        $adjustment->set_value($value);

			if($self->{'nRows'} > $self->{'gKismetApplication'}{'preferences'}->getStringDepth())
			{
				$self->{'clist'}->remove(0);
				$self->{'nRows'}--;
				$adjustment->set_value($value - $step_increment - 1);
			}

			if($value + $page_size >= $upper)
			{
				$adjustment->set_value($value + $step_increment + 1);
			}
			elsif($value == 0 && $page_size + $step_increment >= $upper)
			{
				$adjustment->set_value($upper);
			}

			$adjustment->value_changed();

			$self->{'clist'}->thaw();

		}
	}
}

1;
